@@ -24,16 +24,16 @@ log = logging.getLogger(__name__)
 
 
 class TCPAbridged(TCP):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, proxy: dict):
+        super().__init__(proxy)
         self.is_first_packet = None
 
     def connect(self, address: tuple):
         super().connect(address)
         self.is_first_packet = True
-        log.info("Connected!")
+        log.info("Connected{}!".format(" with proxy" if self.proxy_enabled else ""))
 
-    def send(self, data: bytes):
+    def sendall(self, data: bytes, *args):
         length = len(data) // 4
 
         data = (
@@ -48,20 +48,16 @@ class TCPAbridged(TCP):
 
         super().sendall(data)
 
-    def recv(self) -> bytes or None:
-        length = self.recvall(1)
+    def recvall(self, length: int = 0) -> bytes or None:
+        length = super().recvall(1)
 
         if length is None:
             return None
 
         if length == b"\x7f":
-            length = self.recvall(3)
+            length = super().recvall(3)
 
             if length is None:
                 return None
 
-        length = int.from_bytes(length, "little") * 4
-
-        packet = self.recvall(length)
-
-        return packet
+        return super().recvall(int.from_bytes(length, "little") * 4)
