@@ -19,6 +19,7 @@
 import logging
 import platform
 import threading
+import time
 from datetime import timedelta, datetime
 from hashlib import sha1, sha256
 from io import BytesIO
@@ -31,7 +32,7 @@ from pyrogram import __copyright__, __license__, __version__
 from pyrogram.api import functions, types, core
 from pyrogram.api.all import layer
 from pyrogram.api.core import Message, Object, MsgContainer, Long, FutureSalt, Int
-from pyrogram.api.errors import Error
+from pyrogram.api.errors import Error, InternalServerError
 from pyrogram.connection import Connection
 from pyrogram.crypto import AES, KDF
 from .internals import MsgId, MsgFactory, DataCenter
@@ -61,7 +62,7 @@ class Session:
 
     INITIAL_SALT = 0x616e67656c696361
     NET_WORKERS = 1
-    WAIT_TIMEOUT = 30
+    WAIT_TIMEOUT = 15
     MAX_RETRIES = 5
     ACKS_THRESHOLD = 8
     PING_INTERVAL = 5
@@ -85,9 +86,9 @@ class Session:
     def __init__(self,
                  dc_id: int,
                  test_mode: bool,
-                 proxy: type,
+                 proxy: dict,
                  auth_key: bytes,
-                 api_id: str,
+                 api_id: int,
                  is_cdn: bool = False,
                  client: pyrogram = None):
         if not Session.notice_displayed:
@@ -122,8 +123,6 @@ class Session:
         self.is_connected = Event()
 
     def start(self):
-        terms = None
-
         while True:
             try:
                 self.connection.connect()
@@ -141,7 +140,7 @@ class Session:
                 self.next_salt_thread.start()
 
                 if not self.is_cdn:
-                    terms = self._send(
+                    self._send(
                         functions.InvokeWithLayer(
                             layer,
                             functions.InitConnection(
@@ -150,10 +149,10 @@ class Session:
                                 self.SYSTEM_VERSION,
                                 self.APP_VERSION,
                                 "en", "", "en",
-                                functions.help.GetTermsOfService(),
+                                functions.help.GetConfig(),
                             )
                         )
-                    ).text
+                    )
 
                 self.ping_thread = Thread(target=self.ping, name="PingThread")
                 self.ping_thread.start()
@@ -161,6 +160,9 @@ class Session:
                 log.info("Connection inited: Layer {}".format(layer))
             except (OSError, TimeoutError, Error):
                 self.stop()
+            except Exception as e:
+                self.stop()
+                raise e
             else:
                 break
 
@@ -168,8 +170,6 @@ class Session:
 
         log.debug("Session started")
 
-        return terms
-
     def stop(self):
         self.is_connected.clear()
 
@@ -190,6 +190,9 @@ class Session:
         for i in range(self.NET_WORKERS):
             self.recv_queue.put(None)
 
+        for i in self.results.values():
+            i.event.set()
+
         log.debug("Session stopped")
 
     def restart(self):
@@ -309,8 +312,10 @@ class Session:
                 break
 
             try:
-                self._send(functions.PingDelayDisconnect(0, self.PING_INTERVAL + 15), False)
-            except (OSError, TimeoutError):
+                self._send(functions.PingDelayDisconnect(
+                    0, self.WAIT_TIMEOUT + 10
+                ), False)
+            except (OSError, TimeoutError, Error):
                 pass
 
         log.debug("PingThread stopped")
@@ -339,7 +344,7 @@ class Session:
 
             try:
                 self.current_salt = self._send(functions.GetFutureSalts(1)).salts[0]
-            except (OSError, TimeoutError):
+            except (OSError, TimeoutError, Error):
                 self.connection.close()
                 break
 
@@ -394,14 +399,19 @@ class Session:
             else:
                 return result
 
-    def send(self, data: Object):
-        for i in range(self.MAX_RETRIES):
-            self.is_connected.wait()
+    def send(self, data: Object, retries: int = MAX_RETRIES):
+        self.is_connected.wait(self.WAIT_TIMEOUT)
 
-            try:
-                return self._send(data)
-            except (OSError, TimeoutError):
-                log.warning("Retrying {}".format(type(data)))
-                continue
-        else:
-            return None
+        try:
+            return self._send(data)
+        except (OSError, TimeoutError, InternalServerError) as e:
+            if retries == 0:
+                raise e from None
+
+            (log.warning if retries < 3 else log.info)(
+                "{}: {} Retrying {}".format(
+                    Session.MAX_RETRIES - retries,
+                    datetime.now(), type(data)))
+
+            time.sleep(0.5)
+            return self.send(data, retries - 1)
